package guestbook;

import guestbook.*;

import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.util.*;
import com.opencms.template.*;
import com.opencms.defaults.*;

import java.sql.*;
import java.text.*;
import java.util.*;

/**
 * <p>Title: Guestbook</p>
 * <p>Description: Little helper building a questbook, storing entries in DB</p>
 * <p>Copyright: Freeware - NO WARRANTY</p>
 * <p>Company: Gymnastics Association Bauma, Switzerland</p>
 * @author Cyrill Brunschwiler - BRU
 * @version 1.0
 */

public class GuestbookContentDefinition extends A_CmsContentDefinition implements I_CmsContent {

    // Setup values of each guestbook entry 
    private int m_id = -1;
    private Timestamp m_timestamp = null;
    private String m_author = "";
    private String m_text   = "";
    private String m_email  = "";
    private String m_link   = "";

    // Setup table name and property for id generation/storing
    private static final String C_TABLENAME       = "guestbook_entry";
    private static final String C_TABLE_GUESTBOOK = "GUESTBOOK_ENTRY";

    // Setup prepared statements for table manipulation
    private static final String C_INSERT_RECORD   = "insert into " + C_TABLENAME + " values(?,?,?,?,?,?)";
    private static final String C_DELETE_RECORD   = "delete from " + C_TABLENAME + " where id=?";
    private static final String C_SELECT_PARTS    = "select * from " + C_TABLENAME + " order by timestamp desc limit ?,?";
    private static final String C_SELECT_ALL      = "select * from " + C_TABLENAME + " order by timestamp desc";
    private static final String C_COUNT           = "select count(*) as nr_rows from " + C_TABLENAME;

    // Setup misc values
    private static Locale m_locale = new Locale("Default", "" + Locale.getDefault());
    private static String c_pool = null;
    
    /**
     * Construct a new guestbook entry
     * 
     * @param author The entry authors name
     * @param text The text written by the author
     * @param email The autors email address (optionally = "")
     * @param link A link pointing on autors ressources (optionally = "")
     * @throws CmsException if any error occurs while assigning a new id or while validating the input
     */
    public GuestbookContentDefinition(String author, String text, String email, String link)
            throws CmsException {
        
        this(com.opencms.dbpool.CmsIdGenerator.nextId(c_pool, C_TABLE_GUESTBOOK),
             author, text, email, link,
             new Timestamp(new java.util.Date().getTime()));
    }
    /**
     * Construct a guestbook entry
     * 
     * @param id The elements id
     * @param author The entry authors name
     * @param text The text written by the author
     * @param email The autors email address (optionally = "")
     * @param link A link pointing on autors ressources (optionally = "")
     * @throws CmsException if any error occurs while assigning a new id or while validating the input
     */
    public GuestbookContentDefinition(int id, String author, String text, String email, String link, Timestamp timestamp) {

        m_author = author;
        m_text = text;
        m_email = email;
        m_link = link;
        m_timestamp = timestamp;
        m_id = id;
    }
    
    public String getUniqueId(CmsObject cms) {    
        return "" + m_id;
    }
    
    public String getAuthor() {
        return m_author;
    }
    
    public String getText() {
        return m_text;
    }
    
    public String getEmail() {
        return m_email;
    }
    
    public String getLink() {
        return m_link;
    }
    
    /**
     * Build my own date like: Monday, 4. January 2003, 20:33
     * 
     * @return formatted date
     */
    public String getDate() {   
        DateFormat sdf = new SimpleDateFormat("EEEE, d. MMMM yyyy, HH:mm", m_locale);
        
        return sdf.format(new java.util.Date(m_timestamp.getTime()));
    }
    
    /**
     * Called by OpenCMS if any modul param changes
     * 
     * @param cms The actual cms object
     * @throws CmsException if an error occurs while accessing the registry
     */
    public static void moduleParameterWasUpdated(CmsObject cms) throws CmsException {
        c_pool = OpenCms.getRegistry().getModuleParameterString("guestbook", "poolname");
    }

    /**
     * Write a new element into table
     * 
     * @param cms An instance of CmsObject
     * @throws CmsException if an error occurs while writing data to DB
     */
    public void write(CmsObject cms) throws CmsException {

        Connection con = null;
        
        // clear the cache to make changes visible
        Util.clearCache(cms);

        try {
            // write a record to database
            con = DriverManager.getConnection(c_pool);
            PreparedStatement pst = con.prepareStatement(C_INSERT_RECORD);
            pst.setInt(1, m_id);
            pst.setString(2, m_author);
            pst.setString(3, m_text);
            pst.setString(4, m_email);
            pst.setString(5, m_link);
            pst.setTimestamp(6, m_timestamp);  
            pst.execute();
            pst.close();
        }
        catch(Exception e) {
            throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
        finally{
            try {
                con.close();
            }
            catch (Exception e) {
                throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(), CmsException.C_SQL_ERROR, e);        
            }
        }
    }
 
    /**
     * Deletes cms object (record) from table
     * 
     * @param cms The object to delete
     * @throws CmsException if any error occurs during DB access
     */
    public void delete(CmsObject cms) throws CmsException {
        Connection con = null;
        
        // clear the cache to make changes visible
        Util.clearCache(cms); 
        
        try {
            // delete a record from database
            con = DriverManager.getConnection(c_pool);
            PreparedStatement pst = con.prepareStatement(C_DELETE_RECORD);
            pst.setInt(1, m_id);
            pst.execute();
            pst.close();
        }
        catch(Exception e) {
            throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
        finally {
            try {
                con.close();
            }
            catch (Exception e) {
                throw new CmsException("[" + this.getClass().getName() + "] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
            }
		}
    }

    /**
     * Gets some records from the table. eg. 10-13
     * 
     * @param from The row number of the first record in the part
     * @param limit The max number of records
     * @return a vector filled with the requested content objects
     * @throws CmsException if any error occurs during db access
     */
    public static Vector getEntries(int from, int limit) throws CmsException {
        Connection con = null;
        ResultSet rs = null;
        Vector vecEntries = new Vector();

        try {
            con = DriverManager.getConnection(c_pool);
            PreparedStatement pst = con.prepareStatement(C_SELECT_PARTS);
            pst.setInt(1, from);
            pst.setInt(2, limit);
            rs = pst.executeQuery();
            pst.close();

            while (rs.next()) {
                vecEntries.add(new GuestbookContentDefinition(rs.getInt(1),
                                                              rs.getString(2),
                                                              rs.getString(3),
                                                              rs.getString(4),
                                                              rs.getString(5),
                                                              rs.getTimestamp(6)));
            }
        }
        catch(Exception e) {
            throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
        finally {
            try {
                con.close();
            }
            catch (Exception e) {
                throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
            }
        }
        return vecEntries;
    }
    
    /**
     * Gets all records from the table.
     * 
     * @return a vector cotaining all content objects
     * @throws CmsException if any error occurs during db access 
     */
    public static Vector getAllEntries() throws CmsException {
        Connection con = null;
        ResultSet rs = null;
        Vector vecEntries = new Vector();
        
        try {
            con = DriverManager.getConnection(c_pool);
            PreparedStatement pst = con.prepareStatement(C_SELECT_ALL);
            rs = pst.executeQuery();
            pst.close();
            
            while (rs.next()) {
                vecEntries.add(new GuestbookContentDefinition(rs.getInt(1),
                                                              rs.getString(2),
                                                              rs.getString(3),
                                                              rs.getString(4),
                                                              rs.getString(5),
                                                              rs.getTimestamp(6)));
            }
        }
        catch(Exception e) {
            throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
        finally {
            try {
                con.close();
            }
            catch (Exception e) {
                throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
            }
		}
        return vecEntries;
    }
    
    /**
     * Counts the records in the table.
     * 
     * @return The number of records or 0 if an error occured
     * @throws CmsException if any error occurs during db access 
     */
    public static int getNrRecords() throws CmsException {
        Connection con = null;
        ResultSet rs = null;
        
        try {
            con = DriverManager.getConnection(c_pool);
            PreparedStatement pst = con.prepareStatement(C_COUNT);
            rs = pst.executeQuery();
            pst.close();

            if (rs.next())
                return rs.getInt(1);
            else
                return 0;
        }
        catch(Exception e) {
            throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
        }
        finally {
            try {
                con.close();
            }
            catch (Exception e) {
                throw new CmsException("[GuestbookContentDefinition] " + e.getMessage(), CmsException.C_SQL_ERROR, e);
            }
        }
    }
    
    /**
     *  This code is executed on load-time of this class
     */
    static {
        try {
            // set the pool
            moduleParameterWasUpdated(null);
        }
        catch(CmsException  exc) {
            System.err.println("[GuestbookContentDefinition]: error loading pool.");        
        }
    }
}