package guestbook;

import com.opencms.template.*;
import com.opencms.core.*;
import com.opencms.file.*;

import java.util.*;

/**
 * <p>Title: Guestbook</p>
 * <p>Description: Building a questbook, reading and storing entries in DB</p>
 * <p>Copyright: Freeware - NO WARRANTY</p>
 * <p>Company: Gymnastics Association Bauma ZH, Switzerland</p>
 * @author Cyrill Brunschwiler - BRU
 * @version 1.0
 */

public class GuestbookTemplate extends CmsXmlTemplate {

    /**
     * Gets the content of a defined section in a given template file and its subtemplates
     * with the given parameters.
     *
     * @see getContent(CmsObject cms, String templateFile, String elementName, Hashtable parameters)
     * 
     * @param cms CmsObject Object for accessing system resources.
     * @param templateFile Filename of the template file.
     * @param elementName Element name of this template in our parent template.
     * @param parameters Hashtable with all template class parameters.
     * @param templateSelector template section that should be processed.
     * @return processed template
     * @throws CmsException if any error occurs
     */
    public byte[] getContent(CmsObject cms,
                             String templateFile,
                             String elementName,
                             Hashtable parameters,
                             String templateSelector)
            throws CmsException {

        if(com.opencms.boot.I_CmsLogChannels.C_PREPROCESSOR_IS_LOGGING && A_OpenCms.isLogging() && C_DEBUG ) {
            A_OpenCms.log(C_OPENCMS_DEBUG, "[CmsXmlTemplate] getting content of element " + ((elementName == null) ? "<root>" : elementName));
            A_OpenCms.log(C_OPENCMS_DEBUG, "[CmsXmlTemplate] template file is: " + templateFile);
            A_OpenCms.log(C_OPENCMS_DEBUG, "[CmsXmlTemplate] selected template section is: " + ((templateSelector == null) ? "<default>" : templateSelector));
        }

        CmsXmlTemplateFile xmlTemplateDocument = getOwnTemplateFile(cms, templateFile, elementName, parameters, templateSelector);

        if(templateSelector == null || "".equals(templateSelector))
            templateSelector = (String)parameters.get(C_FRAME_SELECTOR);

        parameters.put("_ELEMENT_", elementName);

        return startProcessing(cms, xmlTemplateDocument, elementName, parameters, templateSelector);
    }

    public Object getGuestbook(CmsObject cms,
                               String tagcontent,
                               A_CmsXmlContent doc,
                               Object userObject)
            throws CmsException {
        
        // prepare some values
        StringBuffer bufferEntries = new StringBuffer();
        Vector vecEntries;
        String action;
        
        int iPage = 0;
        int iRecordsPerPage = -1;

        // get params from request
        Hashtable params = (Hashtable)userObject;
        
        // determine requested action
        action = (String)params.get("action");
        
        // set default action
        if (action == null)
            action = "show";
        
        // user wants to add new entry
        if (action.equals("new")) {
            String author = (String)params.get("author");
            String text = (String)params.get("text");
            String email = (String)params.get("email");
            String link = (String)params.get("link");
            
            /**
             * ADD INPUT VALIDATION HERE
             */
            
            // set optional fields to default values if not set
            if (email == null)
                email = "";
            
            if (link == null)
                link = "";
            
            // add new entry if all fields have possible values
            if (author != null && text != null) {
                GuestbookContentDefinition gcd =
                    new GuestbookContentDefinition(author, text, email, link);
                
                gcd.write(cms);
            }
        }
 
        // if tag is empty show all records
        if (tagcontent != null) {
            try{
                iRecordsPerPage = Integer.parseInt(tagcontent);
                
                // check for param page
                String sPage = (String)params.get("page");
                
                if (sPage != null)
                    iPage = Integer.parseInt(sPage);
            }
            catch(NumberFormatException e) {
                System.err.println("[GuestbookTemplate.getGuestbook] NumberFormatException! " + e.getMessage());
                System.err.println("[GuestbookTemplate.getGuestbook] page: " + (String)params.get("page"));
                System.err.println("[GuestbookTemplate.getGuestbook] tagcontent: " + tagcontent);
            }
        }
        else
            iRecordsPerPage = -1; 
        
        // set min page to zero and and min records to 1
        if (iPage < 0)
            iPage = 0;
        
        if (iRecordsPerPage < 1 && iRecordsPerPage != -1)
            iRecordsPerPage = 1;
        
        // if the whole guestbook is shown on a single page
        if (iRecordsPerPage == -1) {
            vecEntries = GuestbookContentDefinition.getAllEntries();
        }
        else {
            /**
             * how many entries are ot the first page.
             * 
             * Our navigation should look like: 33-21 | 20-11 | 10-1
             */
            int nrRecords = GuestbookContentDefinition.getNrRecords();
            int nrRecordsRest = nrRecords % iRecordsPerPage;

            int from;
            int limit;
            
            // special case for first page
            if (iPage == 0) {
                from = 0;
                limit = iRecordsPerPage + nrRecordsRest;
            }
            else {
                from = nrRecordsRest + iPage*iRecordsPerPage;
                limit = iRecordsPerPage;
            }
            
            vecEntries = GuestbookContentDefinition.getEntries(from, limit);
        }
        
        // fill template
        CmsXmlTemplateFile template = (CmsXmlTemplateFile)doc;
        Iterator it = vecEntries.iterator();
        
        while(it.hasNext()) {
            GuestbookContentDefinition gcd = (GuestbookContentDefinition)it.next();

            template.setData("id", gcd.getUniqueId(null));
            template.setData("author", gcd.getAuthor());
            template.setData("text", gcd.getText());
            template.setData("email", gcd.getEmail());
            template.setData("link", gcd.getLink());
            template.setData("date", gcd.getDate());
            
            // "this" and "userObject" have to be passed here altough this is not said in the documentation!!!
            String temp = template.getProcessedDataValue("guestbookentry", this, userObject);
            bufferEntries.append(temp);
        }
        
        return bufferEntries.toString();
    }
    
    public Object getNavigation(CmsObject cms,
                                String tagcontent,
                                A_CmsXmlContent doc,
                                Object userObject)
            throws CmsException {

        // prepare some values
        StringBuffer bufferNav = new StringBuffer();
        int iRecordsPerPage = -1;

        // if tag is empty show all records
        if (tagcontent != null) {
            try{
                iRecordsPerPage = Integer.parseInt(tagcontent);
            }
            catch(NumberFormatException e) {
                System.err.println("[GuestbookTemplate.getGuestbook] NumberFormatException! " + e.getMessage());
                System.err.println("[GuestbookTemplate.getGuestbook] tagcontent: " + tagcontent);
            }
        }
        else
            iRecordsPerPage = -1; 
        
        if (iRecordsPerPage < 1 && iRecordsPerPage != -1)
            iRecordsPerPage = 1;
        
        /**
         * how many entries are ot the first page.
         * 
         * Our navigation should look like: 33-21 | 20-11 | 10-1
         */
        int nrRecords = GuestbookContentDefinition.getNrRecords();
        int nrRecordsRest = nrRecords % iRecordsPerPage;

        // fill template
        CmsXmlTemplateFile template = (CmsXmlTemplateFile)doc;
        
        // in case of records per page > 0
        for (int i=0; i < nrRecords / iRecordsPerPage; i++) {
            
            int from;
            int to;
             
            // special case for first page
            if (i == 0) {
                from = nrRecords;
                to = from - nrRecordsRest - iRecordsPerPage + 1; 
            }
            else {
                from = nrRecords - nrRecordsRest - i*iRecordsPerPage;
                to = from - iRecordsPerPage + 1;
            }
            
            template.setData("navhref", "page=" + i);
            template.setData("navname", from + "-" + to);

            // "this" and "userObject" have to be passed here altough this is not said in the documentation!!!
            String temp = template.getProcessedDataValue("navigation", this, userObject);
            bufferNav.append(temp);
        }
        
        // in case of showing all records
        if (iRecordsPerPage == -1) {
            template.setData("navhref", "page=0");
            template.setData("navname", nrRecords + "-1");

            // "this" and "userObject" have to be passed here altough this is not said in the documentation!!!
            String temp = template.getProcessedDataValue("navigation", this, userObject);
            bufferNav.append(temp);
        }

        return bufferNav.toString();
    }
    
    public CmsCacheDirectives getCacheDirectives(CmsObject cms,
                                                 String templateFile,
                                                 String elementName,
                                                 Hashtable parameters,
                                                 String templateSelector) {
        
        // setup values
        CmsCacheDirectives ccd;
        
        // determine requested action
        String action = (String)parameters.get("action");
        
        // set default action
        if (action == null)
            action = "show";
        
        if (action.equals("new")) {
            // set uncacheable
            ccd = new CmsCacheDirectives(false);
        }
        else {
            // what paras is the cache dependent to
            Vector vecParams = new Vector();
            vecParams.add("page");
            
            // build no directive using the cache params
            ccd = new CmsCacheDirectives(true);
            ccd.setCacheParameters(vecParams);
        }
        return ccd;
    }
}
